==> univention-management-console-module-ucr_9.0.0-6A~5.0.0.202105111351.tar.gz: univention-management-console-module-ucr-9.0.0/umc/python/ucr/__init__.py <==
#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# Univention Management Console
#  module: manages Univention Config Registry variables
#
# Copyright 2006-2021 Univention GmbH
#
# https://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <https://www.gnu.org/licenses/>.

from io import StringIO

from univention.lib.i18n import Translation
from univention.management.console.base import Base, UMC_Error
from univention.management.console.config import ucr

from univention.management.console.modules.decorators import simple_response, sanitize
from univention.management.console.modules.sanitizers import PatternSanitizer, ChoicesSanitizer, DictSanitizer, StringSanitizer

from univention.config_registry import handler_set, handler_unset, ConfigRegistry, validate_key
from univention.config_registry_info import ConfigRegistryInfo, Variable

import univention.info_tools as uit

_ = Translation('univention-management-console-module-ucr').translate


from typing import Any, Dict, List, Union  # noqa F401


class UCRKeySanitizer(StringSanitizer):

	def _sanitize(self, value: str, name: str, further_arguments: List[Any]) -> Union[str, None]:
		"""
		sanitizing UCR keys

		:param value: Value to be sanitized
		:param name: name of the key
		:param further_arguments: List of further arguments
		:return: sanitized value or None if an Error is raised
		"""
		value = super(UCRKeySanitizer, self)._sanitize(value, name, further_arguments)
		b = StringIO()
		if not validate_key(value, b):
			error_message = b.getvalue()
			pre_error_message = _('A valid UCR variable name must contain at least one character and can only contain letters, numerals, "/", ".", ":", "_" and "-".')
			self.raise_validation_error(f'{pre_error_message} {error_message}')
			return
		return value


class Instance(Base):

	def init(self):
		# set the language in order to return the correctly localized labels/descriptions
		uit.set_language(self.locale.language)

	def __create_variable_info(self, options: Dict) -> None:
		"""
		creating variable infos

		:param options: List of options
		"""
		all_info = ConfigRegistryInfo(registered_only=False)
		info = ConfigRegistryInfo(install_mode=True)
		info.read_customized()
		var = Variable()

		# description
		for line in options['descriptions']:
			text = line['text']
			if not text:
				continue
			if 'lang' in line:
				var[f'description[{line["lang"]}]'] = text
			else:
				var['description'] = text
		# categories
		if options['categories']:
			var['categories'] = ','.join(options['categories'])

		# type
		var['type'] = options['type']

		# are there any modifications?
		old_value = all_info.get_variable(options['key'])
		if old_value != var:
			# save
			info.add_variable(options['key'], var)
			info.write_customized()

	def is_readonly(self, key: str) -> bool:
		ucrinfo_system = ConfigRegistryInfo(registered_only=False, load_customized=False)
		var = ucrinfo_system.get_variable(key)
		if var:
			return var.get('readonly') in ('yes', '1', 'true')
		return False

	@sanitize(DictSanitizer({
		'object': DictSanitizer({
			'key': UCRKeySanitizer(required=True),
			'value': StringSanitizer(default=''),
		})
	}))
	def add(self, request: 'Request') -> None:
		# does the same as put
		ucr.load()
		already_set = set(ucr.keys()) & set(v['object']['key'] for v in request.options)
		if already_set:
			raise UMC_Error(_('The UCR variable %s is already set.') % ('", "'.join(already_set)))

		self.put(request)

	@sanitize(DictSanitizer({
		'object': DictSanitizer({
			'key': UCRKeySanitizer(required=True),
			'value': StringSanitizer(default=''),
		})
	}))
	def put(self, request: 'Request') -> None:
		for _var in request.options:
			var = _var['object']
			value = var['value'] or ''
			key = var['key']
			if self.is_readonly(key):
				raise UMC_Error(_('The UCR variable %s is read-only and can not be changed!') % (key,))
			arg = [f'{key}={value}']
			handler_set(arg)

			# handle descriptions, type, and categories
			if 'descriptions' in var or 'type' in var or 'categories' in var:
				self.__create_variable_info(var)
		self.finished(request.id, True)

	def remove(self, request: 'Request') -> None:
		variables = [x for x in [x.get('object') for x in request.options] if x is not None]
		for var in variables:
			if self.is_readonly(var):
				raise UMC_Error(_('The UCR variable %s is read-only and can not be removed!') % (var,))

		handler_unset(variables)
		self.finished(request.id, True)

	def get(self, request: 'Request') -> None:
		ucrReg = ConfigRegistry()
		ucrReg.load()
		ucrInfo = ConfigRegistryInfo(registered_only=False)

		# iterate over all requested variables
		results = []
		for key in request.options:
			info = ucrInfo.get_variable(str(key))
			value = ucrReg.get(str(key))
			if not info and (value or '' == value):
				# only the value available
				results.append({'key': key, 'value': value})
			elif info:
				# info (categories etc.) available
				info['value'] = value
				info['key'] = key
				results.append(info.normalize())
			else:
				# variable not available, request failed
				raise UMC_Error(_('The UCR variable %(key)s could not be found') % {'key': key})
		self.finished(request.id, results)

	def categories(self, request: 'Request') -> None:
		ucrInfo = ConfigRegistryInfo(registered_only=False)
		categories = []
		for id, obj in ucrInfo.categories.items():
			name = obj['name']
			if ucrInfo.get_variables(id):
				categories.append({
					'id': id,
					'label': name
				})
		self.finished(request.id, categories)

	@sanitize(pattern=PatternSanitizer(default='.*'), key=ChoicesSanitizer(['all', 'key', 'value', 'description'], required=True))
	@simple_response
	def query(self, pattern: str, key: str, category: Union[List[str], None] = None) -> Dict:
		'''Returns a dictionary of configuration registry variables
		found by searching for the (wildcard) expression defined by the
		UMCP request. Additionally a list of configuration registry
		categories can be defined.

		The dictionary returned is compatible with the Dojo data store
		format.'''
		variables = []
		if category == 'all':
			# load _all_ config registry variables
			base_info = ConfigRegistryInfo(registered_only=False)
		else:
			# load _all registered_ config registry variables
			base_info = ConfigRegistryInfo()

		if category in ('all', 'all-registered'):
			category = None

		def _match_value(name, var):
			return var.value and pattern.match(var.value)

		def _match_key(name, var):
			return pattern.match(name)

		def _match_description(name, var):
			descr = var.get('description')
			return descr and pattern.match(descr)

		def _match_all(name, var):
			return _match_value(name, var) or _match_description(name, var) or _match_key(name, var)

		func = locals().get(f'_match_{key}')
		for name, var in base_info.get_variables(category).items():
			if func(name, var):
				variables.append({
					'key': name,
					'value': var.value,
					'description': var.get('description', None),
				})

		return variables

==> univention-management-console-module-ucr_9.0.0-8A~5.0.0.202211281417.tar.gz: univention-management-console-module-ucr-9.0.0/umc/python/ucr/__init__.py <==
#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# Univention Management Console
#  module: manages Univention Config Registry variables
#
# Like what you see? Join us!
# https://www.univention.com/about-us/careers/vacancies/
#
# Copyright 2006-2022 Univention GmbH
#
# https://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <https://www.gnu.org/licenses/>.

import re
from io import StringIO
from typing import Pattern

from univention.lib.i18n import Translation
from univention.management.console.base import Base, UMC_Error
from univention.management.console.config import ucr

from univention.management.console.modules.decorators import simple_response, sanitize
from univention.management.console.modules.sanitizers import PatternSanitizer, ChoicesSanitizer, DictSanitizer, StringSanitizer

from univention.config_registry import handler_set, handler_unset, ConfigRegistry, validate_key
from univention.config_registry_info import ConfigRegistryInfo, Variable

import univention.info_tools as uit

_ = Translation('univention-management-console-module-ucr').translate


from typing import Any, Dict, List, Union  # noqa: E402

ONLINE_BASE = 'repository/online'
COMPONENT_BASE = f'{ONLINE_BASE}/component'
DEPRECATED_VARS = ['prefix', 'username', 'password', 'unmaintained', 'port']
DEPRECATED_GEN = [f'{ONLINE_BASE}/{dep}' for dep in DEPRECATED_VARS]
RE_KEY = re.compile(f'{COMPONENT_BASE}/([^/]+)/({"|".join(DEPRECATED_VARS)})')


class UCRKeySanitizer(StringSanitizer):

	def _sanitize(self, value: str, name: str, further_arguments: List[Any]) -> Union[str, None]:
		"""
		sanitizing UCR keys

		:param value: Value to be sanitized
		:param name: name of the key
		:param further_arguments: List of further arguments
		:return: sanitized value or None if an Error is raised
		"""
		value = super(UCRKeySanitizer, self)._sanitize(value, name, further_arguments)
		b = StringIO()
		if not validate_key(value, b):
			error_message = b.getvalue()
			pre_error_message = _('A valid UCR variable name must contain at least one character and can only contain letters, numerals, "/", ".", ":", "_" and "-".')
			self.raise_validation_error(f'{pre_error_message} {error_message}')
			return
		return value


class Instance(Base):

	def init(self):
		# set the language in order to return the correctly localized labels/descriptions
		uit.set_language(self.locale.language)

	def __create_variable_info(self, options: Dict) -> None:
		"""
		creating variable infos

		:param options: List of options
		"""
		all_info = ConfigRegistryInfo(registered_only=False)
		info = ConfigRegistryInfo(install_mode=True)
		info.read_customized()
		var = Variable()

		# description
		for line in options['descriptions']:
			text = line['text']
			if not text:
				continue
			if 'lang' in line:
				var[f'description[{line["lang"]}]'] = text
			else:
				var['description'] = text
		# categories
		if options['categories']:
			var['categories'] = ','.join(options['categories'])

		# type
		var['type'] = options['type']

		# are there any modifications?
		old_value = all_info.get_variable(options['key'])
		if old_value != var:
			# save
			info.add_variable(options['key'], var)
			info.write_customized()

	def is_readonly(self, key: str) -> bool:
		ucrinfo_system = ConfigRegistryInfo(registered_only=False, load_customized=False)
		var = ucrinfo_system.get_variable(key)
		if var:
			return var.get('readonly') in ('yes', '1', 'true')
		return False

	@sanitize(DictSanitizer({
		'object': DictSanitizer({
			'key': UCRKeySanitizer(required=True),
			'value': StringSanitizer(default=''),
		})
	}))
	def add(self, request) -> None:
		# does the same as put
		ucr.load()
		already_set = set(ucr.keys()) & set(v['object']['key'] for v in request.options)
		if already_set:
			raise UMC_Error(_('The UCR variable %s is already set.') % ('", "'.join(already_set)))

		self.put(request)

	@sanitize(DictSanitizer({
		'object': DictSanitizer({
			'key': UCRKeySanitizer(required=True),
			'value': StringSanitizer(default=''),
		})
	}))
	def put(self, request) -> None:
		for _var in request.options:
			var = _var['object']
			value = var['value'] or ''
			key = var['key']
			if self.is_readonly(key):
				raise UMC_Error(_('The UCR variable %s is read-only and can not be changed!') % (key,))
			arg = [f'{key}={value}']
			handler_set(arg)

			# handle descriptions, type, and categories
			if 'descriptions' in var or 'type' in var or 'categories' in var:
				self.__create_variable_info(var)
		self.finished(request.id, True)

	def remove(self, request) -> None:
		variables = [x for x in [x.get('object') for x in request.options] if x is not None]
		for var in variables:
			if self.is_readonly(var):
				raise UMC_Error(_('The UCR variable %s is read-only and can not be removed!') % (var,))

		handler_unset(variables)
		self.finished(request.id, True)

	def get(self, request) -> None:
		ucrReg = ConfigRegistry()
		ucrReg.load()
		ucrInfo = ConfigRegistryInfo(registered_only=False)

		# iterate over all requested variables
		results = []
		for key in request.options:
			info = ucrInfo.get_variable(str(key))
			value = ucrReg.get(str(key))
			if not info and (value or '' == value):
				# only the value available
				results.append({'key': key, 'value': value})
			elif info:
				# info (categories etc.) available
				info['value'] = value
				info['key'] = key
				results.append(info.normalize())
			else:
				# variable not available, request failed
				raise UMC_Error(_('The UCR variable %(key)s could not be found') % {'key': key})
		self.finished(request.id, results)

	def categories(self, request) -> None:
		ucrInfo = ConfigRegistryInfo(registered_only=False)
		categories = []
		for id, obj in ucrInfo.categories.items():
			name = obj['name']
			if ucrInfo.get_variables(id):
				categories.append({
					'id': id,
					'label': name
				})
		self.finished(request.id, categories)

	@sanitize(pattern=PatternSanitizer(default='.*'), key=ChoicesSanitizer(['all', 'key', 'value', 'description'], required=True))
	@simple_response
	def query(self, pattern: str, key: str, category: Union[List[str], None] = None) -> Dict:
		'''Returns a dictionary of configuration registry variables
		found by searching for the (wildcard) expression defined by the
		UMCP request. Additionally a list of configuration registry
		categories can be defined.

		The dictionary returned is compatible with the Dojo data store
		format.'''
		variables = []
		if category == 'all':
			# load _all_ config registry variables
			base_info = ConfigRegistryInfo(registered_only=False)
		else:
			# load _all registered_ config registry variables
			base_info = ConfigRegistryInfo()

		if category in ('all', 'all-registered'):
			category = None

		def _hidden(name: str, reg: Pattern) -> bool:
			if name in DEPRECATED_GEN:
				return True
			return True if reg.fullmatch(name) else False

		def _match_value(name, var):
			return var.value and pattern.match(var.value)

		def _match_key(name, var):
			return pattern.match(name)

		def _match_description(name, var):
			descr = var.get('description')
			return descr and pattern.match(descr)

		def _match_all(name, var):
			return _match_value(name, var) or _match_description(name, var) or _match_key(name, var)

		func = locals().get(f'_match_{key}')
		for name, var in base_info.get_variables(category).items():
			if func(name, var) and not _hidden(name, RE_KEY):
				variables.append({
					'key': name,
					'value': var.value,
					'description': var.get('description', None),
				})

		return variables
